/*******************************************************************************
* Copyright 2009, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
********************************************************************************/

/* This header exports the configuration method(s) defined by this file. */
#include <cyfitter_cfg.h>

/* This header includes all of the necessary type definitions. */
#include <cytypes.h>

/* Standard system includes */
#if !defined(__ARMCC_VERSION)
#include <string.h>
#endif

/* This header contains all of the macros necessary for accessing registers on the device */
#include <cydevice.h>

/* This header contains all of the macros defined based on the users choices in the System DWR as well as instance
   based macros for register access. */
#include <cyfitter.h>

/* This header contains functions needed to enable the digital array */
#include <CyLib.h>

/* The section below contains all of the data used to initialize the device. */
#include <CyDmac.h>

#if defined(__C51__) || defined(__CX51__)
#define CYCODE code
#define CYDATA data
#define CYXDATA xdata
#define CYFAR far
extern void cymemzero(void CYFAR *addr, uint16 size);
extern void cyconfigcpy(uint16 size, const void CYFAR *src, void CYFAR *dest) small;
extern void cyconfigcpycode(uint16 size, const void CYCODE *src, void CYFAR *dest);
#define CYMEMZERO(a,c) cymemzero((a),(c))
#define CYCONFIGCPY(d,s,c) cyconfigcpy((c),(s),(d))
#define CYCONFIGCPYCODE(d,s,c) cyconfigcpycode((c),(s),(d))
#else
#define CYCODE
#define CYDATA
#define CYXDATA
#define CYFAR
#define CYMEMZERO(a,c) cymemset((a),0,(c))
#define CYCONFIGCPY(d,s,c) cymemcpy((d),(s),(c))
#define CYCONFIGCPYCODE(d,s,c) cymemcpy((d),(s),(c))
#endif

#if defined(__GNUC__)
#pragma GCC diagnostic ignored "-Wpointer-to-int-cast"
#endif
#if defined(__GNUC__) || defined(__ARMCC_VERSION)
#define CYPACKED __attribute__ ((packed))
#else
#define CYPACKED
#endif

#define CYDATADECL(name) const uint8 CYCODE name

typedef struct {
	void CYFAR *address;
	uint16 size;
} CYPACKED cfg_memset_t;

typedef struct {
	void CYFAR *dest;
	const void CYCODE *src;
	uint16 size;
} CYPACKED cfg_memcpy_t;

#define CFG_MEMSET_COUNT (sizeof(cfg_memset_list)/sizeof(*cfg_memset_list))
static const cfg_memset_t CYCODE cfg_memset_list [] = {
	/* address, size */
	{(void CYFAR *)(CYDEV_IO_PRT_PRT0_DR), 32},
	{(void CYFAR *)(CYDEV_IO_PRT_PRT3_DR), 16},
	{(void CYFAR *)(CYDEV_IO_PRT_PRT6_DR), 16},
	{(void CYFAR *)(CYDEV_IO_PRT_PRT12_DR), 16},
	{(void CYFAR *)(CYDEV_IO_PRT_PRT15_DR), 16},
	{(void CYFAR *)(CYDEV_UCFG_B0_P0_U0_BASE), 4096},
	{(void CYFAR *)(CYDEV_UCFG_B1_P2_U0_BASE), 2048},
	{(void CYFAR *)(CYDEV_UCFG_DSI0_BASE), 2560},
	{(void CYFAR *)(CYDEV_UCFG_DSI12_BASE), 512},
};

CYDATADECL(cfg_byte_table) [] = {
	0x0B,
	0x00, 0x52, 0x04, /* Base address 0x005200 */
	/* CYDEV_PRTDSI_PRT2_BASE */
	0x10, 0x03, 0x11, 0x05, 
	/* CYDEV_PRTDSI_PRT4_BASE */
	0x20, 0x09, 0x21, 0x0C, 
	0x01, 0x08, 0x01, /* Base address 0x010800 */
	/* CYDEV_UCFG_B0_P4_U1_BASE */
	0xDB, 0x04, 
	0x01, 0x09, 0x08, /* Base address 0x010900 */
	/* CYDEV_UCFG_B0_P4_ROUTE_BASE */
	0x6B, 0xAA, 0x72, 0x80, 0x73, 0x15, 
	/* CYDEV_UCFG_B0_P4_ROUTE_BASE + 0x00000080 */
	0x97, 0x14, 0x9E, 0x80, 0xAF, 0x14, 0xB2, 0x80, 0xE8, 0x0A, 
	0x01, 0x19, 0x04, /* Base address 0x011900 */
	/* CYDEV_UCFG_B1_P4_ROUTE_BASE + 0x00000080 */
	0x9B, 0x2A, 0xAB, 0x01, 0xB7, 0x80, 0xE8, 0x01, 
	0x01, 0x1B, 0x02, /* Base address 0x011B00 */
	/* CYDEV_UCFG_B1_P5_ROUTE_BASE + 0x00000080 */
	0xB3, 0x28, 0xB7, 0x02, 
	0x01, 0x43, 0x01, /* Base address 0x014300 */
	/* CYDEV_UCFG_DSI3_BASE + 0x00000080 */
	0xC2, 0x10, 
	0x01, 0x45, 0x06, /* Base address 0x014500 */
	/* CYDEV_UCFG_DSI5_BASE */
	0x51, 0x10, 0x54, 0x20, 0x59, 0x80, 0x5F, 0x80, 
	/* CYDEV_UCFG_DSI5_BASE + 0x00000080 */
	0xD4, 0x07, 0xD6, 0x04, 
	0x01, 0x46, 0x04, /* Base address 0x014600 */
	/* CYDEV_UCFG_DSI6_BASE + 0x00000080 */
	0x95, 0x20, 0x9D, 0x80, 0x9F, 0x80, 0xA4, 0x20, 
	0x01, 0x47, 0x04, /* Base address 0x014700 */
	/* CYDEV_UCFG_DSI7_BASE + 0x00000080 */
	0xA4, 0x20, 0xAD, 0x90, 0xB3, 0x80, 0xEC, 0x04, 
	0x01, 0x4C, 0x04, /* Base address 0x014C00 */
	/* CYDEV_UCFG_DSI12_BASE + 0x00000080 */
	0xA0, 0x08, 0xA8, 0x20, 0xB4, 0x08, 0xEC, 0x08, 
	0x01, 0x4D, 0x0E, /* Base address 0x014D00 */
	/* CYDEV_UCFG_DSI13_BASE */
	0x50, 0x08, 0x52, 0x02, 0x54, 0x80, 0x5C, 0x10, 
	/* CYDEV_UCFG_DSI13_BASE + 0x00000080 */
	0x90, 0x10, 0xA0, 0x88, 0xA2, 0x02, 0xB0, 0x80, 0xB2, 0x02, 0xB4, 0x10, 0xD4, 0x07, 0xD6, 0x04, 
	0xE8, 0x08, 0xEC, 0x05, 
};
/* BYTES: 138 */

/* IOPINS0_2 Address: CYDEV_IO_PRT_PRT2_DM0 Size (bytes): 8 */
static CYDATADECL(BS_IOPINS0_2_VAL) [] = {
	0x00, 0x0F, 0x0F, 0x00, 0x0F, 0x00, 0x00, 0x00};

/* IOPINS0_4 Address: CYDEV_IO_PRT_PRT4_DM0 Size (bytes): 8 */
static CYDATADECL(BS_IOPINS0_4_VAL) [] = {
	0x00, 0x0F, 0x0F, 0x00, 0x0F, 0x00, 0x00, 0x00};

/* IOPINS0_5 Address: CYDEV_IO_PRT_PRT5_DM0 Size (bytes): 8 */
static CYDATADECL(BS_IOPINS0_5_VAL) [] = {
	0x00, 0x20, 0x20, 0x00, 0x00, 0x00, 0x40, 0x00};


#if defined(__C51__) || defined(__CX51__)
extern void cfg_write_bytes_code(void CYCODE *table);
extern void cfg_write_bytes(void CYFAR *table);
#else
/* For 32-bit little-endian architectures */
static void cfg_write_bytes32(void)
{
	const uint8 *tablePtr = cfg_byte_table;
	int recordCount = *tablePtr++;

	do
	{
		uint8 *base = *((uint8 **)tablePtr);	/* Read upper address and pair count */
		int pairCount = (int)base & 0x000000FF;
		base -= pairCount;
		tablePtr += sizeof(uint8 *);
		do
		{
			uint8 offset = *tablePtr++;			/* LSB of address */
			CY_SET_XTND_REG8(base + offset, *tablePtr++);
		}
		while (--pairCount);
	}
	while (--recordCount);
}
#endif

static void ClockSetup(void)
{
	uint32 CYDATA timeout;
	CY_SET_XTND_REG24((void CYFAR *)(CYDEV_CLKDIST_ACFG0_CFG0), 0x180095);
	CY_SET_REG8((void CYXDATA *)(CYDEV_SLOWCLK_ILO_CR0), 0x02);
	CY_SET_REG8((void CYXDATA *)(CYDEV_CLKDIST_CR), 0x08);
	CY_SET_REG8((void CYXDATA *)(CYDEV_FASTCLK_IMO_CR), 0x03);
	CY_SET_REG8((void CYXDATA *)(CYDEV_MFGCFG_IMO_TR1), (CY_GET_XTND_REG8(CYDEV_FLSHID_BASE + 0x108)));
	CY_SET_XTND_REG16((void CYFAR *)(CYDEV_FASTCLK_PLL_P), 0x0008);
	CY_SET_REG8((void CYXDATA *)(CYDEV_CLKDIST_CR), 0x08);
	CY_SET_REG8((void CYXDATA *)(CYDEV_FASTCLK_PLL_CFG0), 0x51);
	for (timeout = 1024; timeout && !(CY_GET_REG8(CYDEV_FASTCLK_PLL_SR) & 0x01u); timeout--) { }
	if (timeout == 0)
		return;
	CY_SET_XTND_REG16((void CYFAR *)(CYDEV_CLKDIST_MSTR0), 0x0103);
	CY_SET_REG8((void CYXDATA *)(CYDEV_CLKDIST_MSTR0), 0x00);
	CY_SET_REG8((void CYXDATA *)(CYDEV_CLKDIST_BCFG0), 0x00);
	CY_SET_REG8((void CYXDATA *)(CYDEV_CLKDIST_BCFG2), 0x48);
	CY_SET_REG8((void CYXDATA *)(CYDEV_CLKDIST_LD), 0x02);

	CY_SET_REG8((void CYXDATA *)(CYDEV_PM_ACT_CFG1), ((CY_GET_REG8(CYDEV_PM_ACT_CFG1) | 0x01)));
}

/* Analog API Functions */
void AnalogSetDefault(void)
{
	CY_SET_REG8(CYDEV_IO_PRT_PRT5_AG, 0x40);
	CY_SET_REG8(CYDEV_IO_PRT_PRT12_AG, 0xAA);
	CY_SET_REG8(CYDEV_ANAIF_RT_DSM0_SW0, 0x04);
	CY_SET_REG8(CYDEV_ANAIF_RT_DSM0_SW3, 0x40);
	CY_SET_REG8(CYDEV_ANAIF_RT_BUS_SW0, 0x04);
	CY_SET_REG8(CYDEV_ANAIF_CFG_PUMP_CR0, 0x66);
	CY_SET_REG8(CYDEV_ANAIF_CFG_PUMP_CR1, 0x46);
}

#define CY_AMUX_UNUSED CYDEV_BOOST_SR

/* Digital routing enable Functions */
static void EnableDigitalRouting() {
	CY_SET_XTND_REG8(CYDEV_UCFG_BCTL0_BANK_CTL, CY_GET_XTND_REG8(CYDEV_UCFG_BCTL0_BANK_CTL) | 0x02);
	CY_SET_XTND_REG8(CYDEV_UCFG_BCTL1_BANK_CTL, CY_GET_XTND_REG8(CYDEV_UCFG_BCTL1_BANK_CTL) | 0x02);
}

/* This function is called by the start-up code for the selected device. */
void cyfitter_cfg(void)
{
	uint8 CYDATA i;

	CY_SET_REG8((void CYXDATA *)(CYDEV_CACHE_CR), ((CYDEV_INSTRUCT_CACHE_ENABLED) ? 0x50 : 0x40));
	CY_SET_REG8((void CYXDATA *)(CYDEV_PHUB_CH0_BASIC_CFG), 0x00);
	CY_SET_REG8((void CYXDATA *)(CYDEV_PHUB_CH1_BASIC_CFG), 0x00);

	CY_SET_REG8((void CYXDATA *)(CYDEV_ANAIF_CFG_MISC_CR0), 0x01);

#if (CYDEV_DEBUGGING_ENABLE)
	CY_SET_XTND_REG8(CYDEV_MFGCFG_MLOGIC_DBG_EN_DBG_EN, 0x01);
#endif /* CYDEV_DEBUGGING_ENABLE */

	for (i = 0; i < CFG_MEMSET_COUNT; i++)
	{
		const cfg_memset_t CYCODE * CYDATA ms = &cfg_memset_list[i];
		CYMEMZERO(ms->address, ms->size);
	}

	cfg_write_bytes_code(cfg_byte_table);

	EnableDigitalRouting();

	CyEnableDigitalArray();

	CYCONFIGCPYCODE((void CYFAR *)(CYDEV_IO_PRT_PRT2_DM0), (void CYCODE *)(BS_IOPINS0_2_VAL), 8);
	CYCONFIGCPYCODE((void CYFAR *)(CYDEV_IO_PRT_PRT4_DM0), (void CYCODE *)(BS_IOPINS0_4_VAL), 8);
	CYCONFIGCPYCODE((void CYFAR *)(CYDEV_IO_PRT_PRT5_DM0), (void CYCODE *)(BS_IOPINS0_5_VAL), 8);
	CY_SET_REG8((void CYXDATA *)(CYDEV_INTC_CLOCK_EN), 0x01);
	/* BYTES WRITTEN: 25 */

	CY_SET_REG8((void CYXDATA *)(CYDEV_CACHE_CR), ((CYDEV_INSTRUCT_CACHE_ENABLED) ? 0x10 : 0x00));
	ClockSetup();
	CY_SET_REG8((void CYXDATA *)(CYDEV_CACHE_CR), ((CYDEV_INSTRUCT_CACHE_ENABLED) ? 0x50 : 0x40));

	AnalogSetDefault();
}

/* [] END OF FILE */
